#!/usr/bin/python3
#myservice_description: global notificator/in development
#
#   I need to combine with the advanced RPIHAT (running invisibly in screen)
#
#
import notify2
#sudo aptitude install python3-notify2

import time
#from multiprocessing import Queue
from queue import LifoQueue, Queue, Empty, Full
import os

import glob
import subprocess

from fire import Fire

from gtts import gTTS
from pydub import AudioSegment
from pydub.playback import play

from pydub.utils import make_chunks

import re
from notifator import notihat





class ScreenTool:
    servlist=[]
    def __init__(self):
        res=self.check_myservice()
        print("i... "+str(len(res))+" screens seen for the user")
        print( "i... : {}\n".format(res ) )


    def check_myservice(self):
        #### CMD="screen -ls "+self.screenname
        user_name = os.getenv('USER')
        res=glob.glob( "/var/run/screen/S-"+user_name+"/*" )
        res=[os.path.basename(x) for x in res] # 4321.myservice_
        shortres=["".join(x.split(".")[1:]) for x in res]
        self.servlist=res
        return res


    def run_screen(self, name, cmd):
        self.screenname=name
        print("i... starting screen")
        CMD=cmd
        print("D...  --- ",CMD)
        CMD='/usr/bin/screen -dmS '+self.screenname+' bash -c  "'+CMD+'"'
        print( CMD )
        ok=False
        #try:
        subprocess.check_call( CMD, shell=True )
        ok=True
        print("i...      run ok ")
        print("i...  screen STARTED")
        #except:
        #print("X...      xxx... screen  PROBLEM")






class NotifyClass(object):

    def __init__(self, dest, message, timeshow = 10, color = 1.1):
        print("D... notifator target/message:",dest,"/",message)
        self.dest = dest
        self.message = message
        self.timeshow = timeshow
        self.color = color


    def audio(self, lang="cs"):
        """
Use google etxt to speech, not too frequently. Creates an mp3 in /tmp and reads it
        """
        #print("@"+self.audio.__name__, end='')
        t = re.compile("[a-zA-Z0-9.,_-]")
        #unsafe = "abc∂éåß®∆˚˙©¬ñ√ƒµ©∆∫ø"
        unsafe=self.message
        safe = [ch for ch in unsafe if t.match(ch)]
        safe = "".join(safe)

        outfile="/tmp/msg_"+safe+".mp3"
        if not os.path.isfile(outfile):
            print("D... contacting google...")
            tts=gTTS( self.message , lang=lang)
            tts.save(outfile)
        else:
            print("D... taking sound from /tmp/"+outfile)

        # read it
        path_to_file = outfile
        song = AudioSegment.from_mp3(path_to_file)
        play(song+8)

        return self.message

    def dbus(self):
        #print("@"+self.dbus.__name__+' ', end='')
        notify2.init("--Title--")
        #notice = notify2.Notification(self.dest, self.message)
        notice = notify2.Notification( self.message )
        notice.show()
        time.sleep(0.1)
        return self.message

    def hat(self):
        #print("@"+self.hat.__name__+'  ', end='')
        print("D... HAT ON")
        notihat.main(message=self.message,
                     timeshow=self.timeshow,
                     color = self.color)
        return self.message


    def web(self):
        #print("@"+self.web.__name__+'  ', end='')
        return self.message

    def term(self):
        """
        add the user to tty group to be able to listen
        usermod -a -G tty  user
        """

        #print("@"+self.term.__name__+' ', end='')
        msg=self.message
        print(msg)
        msg=msg.replace('"',' ')
        msg=msg.replace('\\',' ')
        msg=msg.replace('|',' ')
        msg=msg.replace('$',' ')
        msg=msg.replace('#',' ')
        msg=msg.replace('`',' ')
        msg=msg.replace("'",' ')
        os.system('echo "'+msg+'" | wall')
        return self.message

#============ this RUNS the message on destination. Call do
    def do( self ):
        return getattr(NotifyClass, self.dest)(self)




def issueall(message):
    """Issues message to all: audio, dbus, hat, term
    """
    q = Queue(6)   # how does this work?

    q.put( NotifyClass('audio',message) )
    #q.put( NotifyClass('dbus' ,'notify-send way - works with gtk') )
    q.put( NotifyClass('dbus' ,   message  ) )
    q.put( NotifyClass('hat'  , 'Init sense hat and display') )
    #q.put( NotifyClass('web'  , 'connect to webpy.py') )
    q.put( NotifyClass('term' , message) )

    #================== RPIHAT PART
    #s=ScreenTool()
    #s.run_screen("sleeptest","sleep 10") # this can contain rpihat...

    while not q.empty():
        res=q.get().do()
        #print("   ", res)

def issue_hat(message, timeshow = 13, color = 1.1):
    """ Issues a message to RpiHAT (or the emulator)
    """
    print("D... issue_hat")
    q = Queue(6)
    print(q)
    q.put( NotifyClass('hat', message, timeshow , color) )
    print("D... hat in queue")
    while not q.empty():
        res=q.get().do()

def issue_sound(message):
    """Issues a message to speech translator
    """
    q = Queue(6)   # how does this work?

    q.put( NotifyClass('audio',message) )
    #q.put( NotifyClass('dbus' ,'notify-send way - works with gtk') )
    #q.put( NotifyClass('dbus' ,   message  ) )
    #q.put( NotifyClass('hat'  , 'Init sense hat and display') )
    #q.put( NotifyClass('web'  , 'connect to webpy.py') )
    #q.put( NotifyClass('term' , message) )

    while not q.empty():
        res=q.get().do()

def issue_dbus(message):
    """Issues a message to DBUS system (crash on RPi)
    """
    q = Queue(6)   # how does this work?

    #q.put( NotifyClass('audio',message) )
    #q.put( NotifyClass('dbus' ,'notify-send way - works with gtk') )
    q.put( NotifyClass('dbus' ,   message  ) )
    #q.put( NotifyClass('hat'  , 'Init sense hat and display') )
    #q.put( NotifyClass('web'  , 'connect to webpy.py') )
    #q.put( NotifyClass('term' , message) )

    while not q.empty():
        res=q.get().do()


#============================================================
#============================================================
#============================================================
#============================================================
#============================================================

if __name__=="__main__":
    Fire( {"a":issueall,
           "s":issue_sound,
           "h":issue_hat,
           "b":issue_dbus})





    ######################## END END END END END END #################
    ######################## END END END END END END #################
    ######################## END END END END END END #################

    #########################
    # TASK:
    #   local and ip applications need to display stuff. messages.
    #   myservice could work as ip port for receiving messages
    #      and everything else is local...
    #  PLAN:
    #  queue external - via -
    # * notify
    # * sense-hat - small num, rolling number
    # * audio - mp3 or voice
    # * webpage
    #https://pymotw.com/2/multiprocessing/communication.html
    #http://askubuntu.com/questions/108764/how-do-i-send-text-messages-to-the-notification-bubbles
    #====== first attempt  d-bus:
    #
    #https://askubuntu.com/questions/38733/how-to-read-dbus-monitor-output/142888
    ############################



#!/usr/bin/env python3
#
###https://projects.raspberrypi.org/en/projects/getting-started-with-the-sense-hat
#
# MERGE OF pix2 and later tricks
#
#
#from sense_emu import SenseHat
#from sense_emu import SenseHat
#
# show_num
# show_number
# make_color
#

try:
    from sense_hat import *
except:
    from sense_emu import *

import time
import sys
import colorsys
import datetime

import os
import subprocess as s

#import os
import signal  # to kill
import datetime #  running until timeout

import argparse

from fire import Fire


#####################################
#  FROM DISPTEMP.PY
####################################
#one row per 5x3 digit - 0 to 9 and T
nums =[1,1,1,1,0,1,1,0,1,1,0,1,1,1,1, # 0
       0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
       1,1,1,0,0,1,0,1,0,1,0,0,1,1,1,
       1,1,1,0,0,1,1,1,1,0,0,1,1,1,1,
       1,0,0,1,0,1,1,1,1,0,0,1,0,0,1,
       1,1,1,1,0,0,1,1,1,0,0,1,1,1,1,
       1,1,1,1,0,0,1,1,1,1,0,1,1,1,1,
       1,1,1,0,0,1,0,1,0,1,0,0,1,0,0,
       1,1,1,1,0,1,1,1,1,1,0,1,1,1,1,
       1,1,1,1,0,1,1,1,1,0,0,1,0,0,1, # 9
       1,1,1,0,1,0,0,1,0,0,1,0,0,1,0] # T

def show_num(val,xd,yd,r,g,b,  sense):
    offset = val * 15
    for p in range(offset,offset + 15):
        if nums[p] == 1:
            xt = p % 3
            yt = (p-offset) // 3
            sense.set_pixel(xt+xd,yt+yd,r,g,b)

def show_number(val,  rgb , sense):
    abs_val = abs(val)
    tens = abs_val // 10
    units = abs_val % 10
    sense.clear()
    if (abs_val > 9): show_num(tens,0,2,rgb[0],rgb[1],rgb[2],  sense)
    show_num(units,4,2,rgb[0],rgb[1],rgb[2], sense)
    if abs_val == 100:
        sense.clear()
        show_num(10,3,2,rgb[0],rgb[1],rgb[2], sense) # 'T' for ton = 100
    if val < 0 :
        for i in range(0,8):
            #sense.set_pixel(i,0,0,0,128)
            sense.set_pixel(i,0, rgb[0],rgb[1],rgb[2])


######################
#
#  color creation ... 0-1.0
#
#########################
def make_color( aa ):
    if aa==-1: # White
        colorRGB=[ 255,255,255 ]
        return colorRGB
    color=colorsys.hsv_to_rgb( aa , 1.0, 0.8 ) # hue,  grayness,  blackness
    colorRGB=[ int(255*i) for i in color ]
    #print( "colorRGB={}".format(colorRGB ))
    return colorRGB




def main(message="Ahoj", timeshow=1, color=1.1, debug=False):
    """PiHAT: send MESSAGE for TIME. If color>1: rotate"""
    message = str(message)
    colorRGB=make_color( float(color) )
    red = (255, 0, 0)
    white = (255, 255, 255)
    black = ( 0,0,0 )

    sense = SenseHat()
    print("D... Sense HAT inited")

    # bb = message
    # 0.05 is fast 0.2 is slow
    speed=0.1* 3/len(message)  # 3 seconds for all
    speed=0.1* timeshow/len(message)  # 3 seconds for all
    speed2=speed
    if speed2<0.04:
        speed2=0.04
        print("D... speed limit 0.04")
    if speed2>0.2:
        speed2=0.2
        print("D... speed limit 0.2")
    speed2=0.1 # I WANT THIS
    print( "D... display speed: wanted {:.2f} will be {:.2f}".format(speed , speed2 ) )

    ###############
    # DISPLAY decisions
    ###############
    SmallDig=False
    Running=True  # alwas possible
    Static=False

    if message.lstrip('-').isdigit():
        print( "D...",message,"is a digit ")
        if len(message.lstrip('-'))<3:
            SmallDig=True
    else:
        print("D... ",message,"not a digit")
    print("D...  bb length is ", len(message) )
    if len(message)==1:
        Static=True

    print("D... Small={} Run={} Stat={}".format(SmallDig, Running, Static) )


    ############################
    # this is pidkilling part... enables new message without clear
    ############################
    PIDFILE="/tmp/rpihat.pid"
    if Static or SmallDig or Running:
        print("i... looking for "+PIDFILE)
        if os.path.isfile( PIDFILE ) :
            #print("i... exists ... killing and deleting")
            with open(PIDFILE,'r') as f:
                pid=f.readlines()[0]#.decode("utf8").rstrip()
                pid=int(pid)
            print("i... file exists, killing ",pid," ", type(pid) )
            try:
                os.kill(pid, signal.SIGTERM) #or signal.SIGKILL
            except:
                print("X... no process like ",pid," nothing killed")
        mypid=os.getpid()
        print("i... writing new PID",mypid)
        with open(PIDFILE,'w') as f:
            f.write( str(mypid)+"\n" )
    #========== pid killing end -----------------------



    ###############
    # DISPLAY:   aa was inicolor
    ###############
    start=datetime.datetime.now()
    sec=0
    if Static or SmallDig:
        if color>1:
            aastart=0.72 # From BLUE TO RED - override
            aa = aastart
        else:
            aastart = color
            aa = aastart
        print("D... static display with sleep: [static/smalldig]", Static, SmallDig,"\n")
        while sec<timeshow:
            colorRGB=make_color(aa)
            if Static and (SmallDig):
                sense.show_letter( message[0], text_colour=colorRGB )
            else: #if SmallDig:
                show_number( int(message) , colorRGB , sense  )
            sec=(datetime.datetime.now()-start).total_seconds()
            if color>1:
                aa=aastart*(timeshow-sec)/timeshow
                if aa<0.:aa=0

            print( "i... {:03.1f}/{} s.   rgb={}".format( sec, timeshow, colorRGB), end="\r" )
            time.sleep( 0.1 )
        Running=False


    #if SmallDig:
    #    print("?small digit display with sleep\n")
    #    #time.sleep( sleep )
    if Running:
        if color > 1.0:
            aastart=0.72 # From BLUE TO RED - override
            aa = aastart
        else:
            aa = color
        deltat = 0
        ndiv = 0
        while sec<timeshow:
            colorrun=make_color(aa)

            print("i.. running message ..{:03.1f}/{} s.,  {}".format(
                sec,
                timeshow,
                colorrun
            ), end="\r")
            sense.show_message( message ,
                            text_colour = colorrun,
                            back_colour=black,
                            scroll_speed=speed2)
            sec=(datetime.datetime.now()-start).total_seconds()
            if deltat == 0:
                deltat = sec
                divs = int(timeshow/deltat)
                if divs == 0: divs = 1
                ndiv=divs # maximal value and go to 0
            ndiv-=1
            if color > 1.0:
                aa = aastart*ndiv/divs # Change color
                if aa<0.:aa=0
    sense.clear()
    print("i... writing NO PID to ",PIDFILE)
    with open(PIDFILE,'w') as f:
        f.write(  "999999\n" )


def hourly():
    while True:
        time.sleep(0.9)
        se = datetime.datetime.now().strftime("%S")
        print(se, end=" ", flush=True)
        if int(se)==0:
            hm = datetime.datetime.now().strftime("%H:%M")
            print("\nD... ",hm )
            mi = datetime.datetime.now().strftime("%M")
            if (int(mi) == 15):
                main(hm, timeshow=13, color=0.2 )
            elif (int(mi) == 30):
                main(hm, timeshow=13, color=0.5 )
            elif (int(mi) == 45):
                main(hm, timeshow=13, color=0.8 )
            elif (int(mi) == 0):
                main(hm, timeshow=13, color=1.1 )
            else:
                main(hm, timeshow=6, color=-1 )
            print()



if __name__ == "__main__":
    Fire( {"main":main,
           "hourly":hourly
    } )
    sys.exit()
